module LS020_SM_INIT(Init_DATA, clk, reset,TX_GO,Ready,CD,ACK);


//Ports
input clk; 					// System clock
input reset; 				// System Reset
input ACK;
inout [15:0] Init_DATA;
inout TX_GO;
inout CD;
output Ready;

//Registers
reg	[15:0]	Init_DATA_REG;
reg	[15:0]	LUT_DATA;
reg	[5:0]	LUT_INDEX; 			//max 64 LUT Eintrge
reg	[1:0]	Init_State;
reg			TX_GO_REG;
reg			Ready;
reg			CD_REG;				// 1 = Command , 0 = Daten
reg			Delay_a;
reg			Delay_b;
reg			Delay_c;
reg			Delay_d;
reg [15:0] 	counter;
reg [13:0] 	delay_count_7 ;
reg [15:0] 	delay_count_50 ;
reg [15:0] 	delay_count_50b ;

// Fr Simulation
//reg [6:0] 	delay_count_7 ;
//reg [8:0] 	delay_count_50 ;
//reg [8:0] 	delay_count_50b ;
		

// Assignments
assign state_clk = counter[3];
assign Init_DATA 	= ( Ready == 1 ) ? 16'bZ: Init_DATA_REG ;
assign TX_GO 		= ( Ready == 1 ) ? 1'bZ : TX_GO_REG;
assign CD			= ( Ready == 1 ) ? 1'bZ : 1 ;


// Always Block


// Init SM
always@(posedge state_clk or negedge reset)
begin
	if(!reset)
	begin
		LUT_INDEX		<=	0;
		Init_State		<=	0;
		TX_GO_REG		<=	0;
	end
	else
	
	begin
		if(LUT_INDEX<LUT_SIZE)
		begin

			case(Init_State)
			0:	begin
					if(!(Delay_a|Delay_b|Delay_c))
						begin
							Init_DATA_REG	<=	LUT_DATA;
							TX_GO_REG		<=	1;
							Init_State	<=	1;
						end
				end
				
			1:	begin
					if(ACK)
						begin
							Init_State	<=	2;
							TX_GO_REG		<=	0;
						end
					end
				
			2:	begin
					LUT_INDEX	<=	LUT_INDEX+1;
					Init_State	<=	0;
				end
				
			endcase						
		end
	end
end



// CD / Ready SM
always@(posedge state_clk)
begin
if(LUT_INDEX > 40)
	begin
		Ready				= 1;
		CD_REG				= 1'bz;
	end
else 
	begin
		Ready				= 0;
		CD_REG				= 1;
	end
end



always @(posedge state_clk) if(LUT_INDEX == 1 | (delay_count_50!=0) ) delay_count_50 <= delay_count_50 + 1;

always @(posedge state_clk) Delay_a <= (delay_count_50!=0 );

always @(posedge state_clk) if(LUT_INDEX == 11 | (delay_count_7!=0)) delay_count_7 <= delay_count_7 + 1;

always @(posedge state_clk) Delay_b <= (delay_count_7!=0);

always @(posedge state_clk) if(LUT_INDEX == 31 | (delay_count_50b!=0)) delay_count_50b <= delay_count_50b + 1;

always @(posedge state_clk) Delay_c <= (delay_count_50b!=0);

//always @(posedge state_clk) if(LUT_INDEX == 34 | (delay_count_50b!=0)) delay_count_50b <= delay_count_50b + 1;

always @(posedge state_clk) Delay_d <= (delay_count_50b!=0);

always @(posedge clk) counter <= counter+1;


// Init LUT
always
begin
	case(LUT_INDEX)
	// Init von http://superkranz.de/christian/S65_Display/DisplayProgramming.html
	I1_0	:	LUT_DATA	<=	16'hFDFD;
	I1_1	:	LUT_DATA	<=	16'hFDFD;
	//50ms Delay / nach State 1
	I2_0	:	LUT_DATA	<=	16'h021A;
	I2_1	:	LUT_DATA	<=	16'hEE04;
	I2_2	:	LUT_DATA	<=	16'h1B04;
	I2_3	:	LUT_DATA	<=	16'hFEFE;
	I2_4	:	LUT_DATA	<=	16'hFEFE;
	I2_5	:	LUT_DATA	<=	16'hEF90;
	I2_6	:	LUT_DATA	<=	16'h4A04;
	I2_7	:	LUT_DATA	<=	16'h7F3F;
	I2_8	:	LUT_DATA	<=	16'hEE04;
	I2_9	:	LUT_DATA	<=	16'h4306;
	//7ms Delay / nach State 11
	I3_0	:	LUT_DATA	<=	16'hEF90;
	I3_1	:	LUT_DATA	<=	16'h0983;
	I3_2	:	LUT_DATA	<=	16'h0800;
	I3_3	:	LUT_DATA	<=	16'h0BAF;
	I3_4	:	LUT_DATA	<=	16'h0A00;
	I3_5	:	LUT_DATA	<=	16'h0500;
	I3_6	:	LUT_DATA	<=	16'h0600;
	I3_7	:	LUT_DATA	<=	16'h0700;
	I3_8	:	LUT_DATA	<=	16'hEF00;
	I3_9	:	LUT_DATA	<=	16'hEE0C;
	I3_10	:	LUT_DATA	<=	16'hEF90;
	I3_11	:	LUT_DATA	<=	16'h0080;
	I3_12	:	LUT_DATA	<=	16'hEFB0;
	I3_13	:	LUT_DATA	<=	16'h4902;
	I3_14	:	LUT_DATA	<=	16'hEF00;
	I3_15	:	LUT_DATA	<=	16'h7F01;
	I3_16	:	LUT_DATA	<=	16'hE181;
	I3_17	:	LUT_DATA	<=	16'hE202;
	I3_18	:	LUT_DATA	<=	16'hE276;
	I3_19	:	LUT_DATA	<=	16'hE183;
	//50ms Delay / nach State 31
	I4_0	:	LUT_DATA	<=	16'h8001;
	I4_1	:	LUT_DATA	<=	16'hEF90;
	I4_2	:	LUT_DATA	<=	16'h0000;
	//Display Ready , 90 Orientierung voller Screen
	T1_0	:	LUT_DATA	<=	16'hEF90;
	T1_1	:	LUT_DATA	<=	16'h0504;
	T1_2	:	LUT_DATA	<=	16'h0800;
	T1_3	:	LUT_DATA	<=	16'h0983;
	T1_4	:	LUT_DATA	<=	16'h0A00;
	T1_5	:	LUT_DATA	<=	16'h0BAF;
	default		:	LUT_DATA	<=	16'h0000;
	
	endcase
end

//	LUT Data Number
parameter	LUT_SIZE	=	41;
//	Init Index
parameter	I1_0	=	0;
parameter	I1_1	=	1;
parameter	I2_0	=	2;
parameter	I2_1	=	3;
parameter	I2_2	=	4;
parameter	I2_3	=	5;
parameter	I2_4	=	6;
parameter	I2_5	=	7;
parameter	I2_6	=	8;
parameter	I2_7	=	9;
parameter	I2_8	=	10;
parameter	I2_9	=	11;
parameter	I3_0	=	12;
parameter	I3_1	=	13;
parameter	I3_2	=	14;
parameter	I3_3	=	15;
parameter	I3_4	=	16;
parameter	I3_5	=	17;
parameter	I3_6	=	18;
parameter	I3_7	=	19;
parameter	I3_8	=	20;
parameter	I3_9	=	21;
parameter	I3_10	=	22;
parameter	I3_11	=	23;
parameter	I3_12	=	24;
parameter	I3_13	=	25;
parameter	I3_14	=	26;
parameter	I3_15	=	27;
parameter	I3_16	=	28;
parameter	I3_17	=	29;
parameter	I3_18	=	30;
parameter	I3_19	=	31;
parameter	I4_0	=	32;
parameter	I4_1	=	33;
parameter	I4_2	=	34;
parameter	T1_0	=	35;
parameter	T1_1	=	36;
parameter	T1_2	=	37;
parameter	T1_3	=	38;
parameter	T1_4	=	39;
parameter	T1_5	=	40;


endmodule
